"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var Mailer_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailDelivery = exports.Mailer = void 0;
/* eslint-disable @typescript-eslint/no-var-requires */
const nodemailer_1 = __importDefault(require("nodemailer"));
const debug_1 = __importDefault(require("debug"));
// @ts-expect-error 7016
const nodemailer_stub_1 = require("nodemailer-stub");
const worker_1 = require("./worker");
const debug = (0, debug_1.default)('hyperworker:mailer');
/*
1. renderer (takes template path, then render takes params)
2. transport (nodemailer)
*/
class MailDelivery {
    constructor(MailerType, details) {
        this.MailerType = MailerType;
        this.details = details;
    }
    deliver() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!worker_1.HyperWorker.logger) {
                throw new Error('Logger is not configured. Did the app boot correctly?');
            }
            const { cyan, bold } = worker_1.HyperWorker.logger.colors;
            const pref = bold(cyan('mailers:'));
            worker_1.HyperWorker.logger.info(`${pref} ${this.MailerType.name} delivering`, this.details);
            return this.MailerType.performNow(this.details);
        });
    }
    deliverLater() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!worker_1.HyperWorker.logger) {
                throw new Error('Logger is not configured. Did the app boot correctly?');
            }
            const { cyan, bold } = worker_1.HyperWorker.logger.colors;
            const pref = bold(cyan('mailers:'));
            worker_1.HyperWorker.logger.info(`${pref} ${this.MailerType.name} delivering later`, this.details);
            return this.MailerType.performLater(this.details);
        });
    }
}
exports.MailDelivery = MailDelivery;
let Mailer = Mailer_1 = class Mailer extends worker_1.HyperWorker {
    // <testing> mailer stub only convenience
    static clearDeliveries() {
        nodemailer_stub_1.interactsWithMail.flushMails();
    }
    static deliveries() {
        return nodemailer_stub_1.interactsWithMail.mails;
    }
    static lastMail() {
        return nodemailer_stub_1.interactsWithMail.lastMail();
    }
    // </testing>
    static clear() {
        this.transport = null;
        this.send = false;
        this.preview = false;
        this.templatesRoot = null;
    }
    static initMailer({ templatesRoot, settings, }) {
        this.templatesRoot = templatesRoot;
        if (settings) {
            this.send = !!settings.send;
            this.preview = !!settings.preview;
            if (settings.delivery === 'test') {
                this.transport = nodemailer_stub_1.stubTransport;
            }
            else if (settings.delivery === 'smtp') {
                this.transport = nodemailer_1.default.createTransport(settings.smtpSettings);
            }
        }
    }
    static setNone() {
        this.send = false;
        this.preview = false;
        this.transport = null;
    }
    static setTest() {
        this.send = true;
        this.preview = false;
        this.transport = nodemailer_stub_1.stubTransport;
    }
    static setDevelopment() {
        this.send = false;
        this.preview = true;
        this.transport = null;
    }
    static setProduction() {
        this.send = true;
        this.preview = false;
    }
    perform({ data }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!worker_1.HyperWorker.logger) {
                throw new Error('Logger is not configured. Did the app boot correctly?');
            }
            if (!Mailer_1.templatesRoot) {
                throw new Error('please set Mailer.templatesRoot');
            }
            // lazy because it takes lots of time to load eagerly
            const Email = require('email-templates');
            debug(`mailer: send settings '%o'`, {
                send: Mailer_1.send,
                preview: Mailer_1.preview,
            });
            const tmpl = new Email({
                views: { root: Mailer_1.templatesRoot, options: { extension: 'ejs' } },
                transport: Mailer_1.transport || { jsonTransport: true },
                send: Mailer_1.send,
                preview: Mailer_1.preview,
                juice: false,
            });
            try {
                debug(`mailer: perform '%o'`, data);
                const res = yield tmpl.send(data);
                const { cyan, bold } = worker_1.HyperWorker.logger.colors;
                const pref = bold(cyan('mailers:'));
                worker_1.HyperWorker.logger.info(`${pref} delivered email to: ${data.message.to}`, res);
                return res;
            }
            catch (e) {
                worker_1.HyperWorker.logger.error(e);
                worker_1.HyperWorker.reportError(e);
            }
        });
    }
    static mail(details, _opts = {}) {
        details.message = Object.assign(Object.assign({}, this.defaults), details.message);
        return new MailDelivery(this, details);
    }
    get logger() {
        return worker_1.HyperWorker.logger;
    }
};
exports.Mailer = Mailer;
Mailer.transport = null;
Mailer.send = false;
Mailer.preview = false;
Mailer.templatesRoot = null;
Mailer.defaults = {
    from: 'Joe <mailer@example.com>',
};
exports.Mailer = Mailer = Mailer_1 = __decorate([
    (0, worker_1.queueAs)('mails')
], Mailer);
