"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HyperWorker = exports.producerOnly = exports.queueAs = void 0;
require("reflect-metadata");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('hyperworker:worker');
const K_QUEUE_NAME = 'hyperworker:queue-name';
const K_PRODUCER_ONLY = 'hyperworker:queue-producer-only';
const queueAs = (queueName) => (target) => {
    return Reflect.defineMetadata(K_QUEUE_NAME, queueName, target);
};
exports.queueAs = queueAs;
const producerOnly = () => (target) => {
    return Reflect.defineMetadata(K_PRODUCER_ONLY, 'true', target);
};
exports.producerOnly = producerOnly;
class HyperWorker {
    static clear() {
        this.logger = null;
        this.backend = null;
        this.inprocess = false;
        this.enqueue = null;
        this.reportError = (_err, _tags) => { };
    }
    static initWorker({ logger }) {
        this.logger = logger;
    }
    static setMode({ inprocess, reportError, }) {
        this.inprocess = inprocess;
        if (reportError) {
            this.reportError = reportError;
        }
    }
    static setTest(reportError) {
        this.setMode({ inprocess: true, reportError });
    }
    static get queueName() {
        return Reflect.getOwnMetadata(K_QUEUE_NAME, this) || this.name.toLowerCase();
    }
    static get producerOnly() {
        return Reflect.getOwnMetadata(K_PRODUCER_ONLY, this) || false;
    }
    static performNow(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const w = new this();
            return w.perform({ data: args });
        });
    }
    static performLater(_args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.inprocess) {
                debug(`${this.name}: perform now '%o'`, _args);
                return this.performNow(_args);
            }
            debug(`${this.name}: perform later '%o'`, _args);
            return this.enqueue(_args, {}); // TODO opts
        });
    }
    static jobCounts() {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.queue.getJobCounts());
        });
    }
    get logger() {
        return HyperWorker.logger;
    }
    perform(_args) {
        // tbd
    }
}
exports.HyperWorker = HyperWorker;
HyperWorker.inprocess = false;
HyperWorker.logger = null;
HyperWorker.backend = null;
HyperWorker.enqueue = null;
HyperWorker.queue = null;
HyperWorker.reportError = (_err, _tags) => { };
